#!/bin/bash

# This script illustrates the protocol for setting up a simulation of a protein
# crystal unit cell.  Source codes for all programs called by this tutorial are
# present in this directory; the script will check for the existence of any
# programs that it needs and prompt the user to compile them if necessary.
#
# All programs in this tutorial may be run independently, and will produce lists
# of options if run with no arguments.  All programs needed for this tutorial
# can be run from the command line; no separate input files are needed.

# In order to run this tutorial, paths to certain AMBER programs must be set:
TLEAP="/cottus/opt/amber11/bin/tleap"

# PART 1
echo "PART 1: Correct the original PDB file (1AHO.pdb) to prevent mistakes"\
     "later on."
echo "        In this case, the PDB file contains two Histidine residues that"\
     "should"
echo "        be protonated in the \"delta\" state (HID), but AMBER, by"\
     "default,"
echo "        protonates HIS residues in the \"epsilon\" state ((HIE).  The"\
     "\"delta\""
echo "        state is preferable for His54 and His64 because it permits"\
     "hydrogen"
echo "        bonding with symmetry-related protein monomers.  In real life,"\
     "this"
echo "        would only be apparent after reconstructing the unit cell in"\
     "Part 2,"
echo "        but we have gone ahead and done it already.  Also, because the"\
     "protein"
echo "        contains numerous disulfide bonds, CYS residue tags must be"\
     "changed"
echo "        to CYX.  Finally, this PDB file contains some alternate "\
     "conformations"
echo "        of certain residues, but AMBER's TLEAP module can only accept"\
     "one."
echo "        The dominant \"A\" conformation was chosen in each case.  The"\
     "file"
echo "        incorporating all of these corrections is provided as c1AHO.pdb."
echo "PART 1: (COMPLETE)"
echo ""

# PART 2
echo "PART 2: Reconstruct the crystal unit cell.  This is done with the"\
     "\"UnitCell\""
echo "        program.  UnitCell will read a PDB file and extract information"\
     "from"
echo "        its CRYST1 and \"REMARK 290 SYMM\" records to determine the"\
     "proper"
echo "        symmetry operations and reconstruct the unit cell."
if [ -e ${AMBERHOME}/bin/UnitCell ] ; then
  if [ -x ${AMBERHOME}/bin/UnitCell ] ; then
    if [ ! -e x1AHO.pdb ] ; then
      ${AMBERHOME}/bin/UnitCell -p c1AHO.pdb -o x1AHO.pdb
      if [ -e x1AHO.pdb ] ; then
        echo "Part 2: (COMPLETE) The unit cell has been reconstructed.  Notice"\
             "that \"HETATM\""
        echo "        records have been changed to \"ATOM\" records; AMBER's"\
             "setup modules"
        echo "        will make no distinction, so the conversion is done now."
        echo ""
      else
        echo "ERROR >> UnitCell should have created a file called"\
             "\"x1AHO.pdb\" given c1AHO.pdb "
        echo "ERROR >> as input, but the file was not created."
        exit
      fi
    else
      echo "Part 2: (COMPLETE)"
      echo ""
    fi
  else
    echo "ERROR >> The program UnitCell exists, but is not executable."
    exit
  fi
else
  echo "ERROR >> The program UnitCell does not exist.  Please compile the "\
       "program"
  echo "ERROR >> if you have not already done so."
  exit
fi

# PART 3
echo "PART 3: Solvate the unit cell.  Looking in the original PDB file (and the"
echo "        primary source, Smith et al., Acta Crystallographica D. 53:551"\
     "(1997)),"
echo "        the crystallization solution contains 0.2M ammonium acetate. "\
     "This is"
echo "        a very mild set of crystallization conditions, and is easily"\
     "modeled"
echo "        with the AMBER antechamber set of programs.  In this particular"\
     "case,"
echo "        we use our own model of ammonium and an acetate model taken from"\
     "Saigal"
echo "        and Pranata (Bioorg. Chem. 25:11, 1997).  The necessary AMBER "\
     "prep and"
echo "        frcmod files are provided (Acetate.prp, Ammonium.prp, and "
echo "        Ammonium.frcmod).  We therefore need to add ammonium acetate"\
     "such that"
echo "        the unit cell will contain a solvent (here, taken to mean the"\
     "non-"
echo "        protein component of the system) that is 0.2M ammonium acetate. "\
     "However,"
echo "        we also must neutralize the overall charge on the system (+4 due"\
     "to its"
echo "        four protein monomers).  To satisfy these constraints, we will"\
     "add"
echo "        three ammonium (+1 charge) seven acetate (-1 charge) ions to the"\
     "system."
echo "        This will be done with the AddToBox program.  In order to "\
     "disperse"
echo "        the ions throughout the simulation cell, we will add them one at"\
     "a time,"
echo "        and require that additional ions be placed at least 6 Angstroms"\
     "from"
echo "        those that have already been placed.  No ions will be allowed "\
     "within"
echo "        3 Angstroms of the lattice proteins.  Additional information"\
     "about the"
echo "        AddToBox program is available in the comments of this script."
echo "        Also, running the AddToBox program with no arguments will print"\
     "a list"
echo "        of command-line input options."

# These variables define the unit cell dimensions, and correspond to the CRYST1
# record of the original PDB file, 1AHO.pdb.
CDX="45.900"
CDY="40.700"
CDZ="30.100"
ALPHA="90.00"
BETA="90.00"
GAMMA="90.00"
XYZ="-X ${CDX} -Y ${CDY} -Z ${CDZ}"
ABC="-al ${ALPHA} -bt ${BETA} -gm ${GAMMA}"

# This is the first AddToBox command, for adding acetate.
if [ ! -e ${AMBERHOME}/bin/AddToBox ] ; then
  echo "ERROR >> The AddToBox program was not found."
  exit
fi
if [ ! -x ${AMBERHOME}/bin/AddToBox ] ; then
  echo "ERROR >> The AddToBox program is not executable."
  exit
fi
if [ ! -e xa1AHO.pdb ] ; then
  echo "PART 3: (PROCEEDING) AddToBox output follows."
  ${AMBERHOME}/bin/AddToBox \
    -c x1AHO.pdb \
    -a Acetate.pdb \
    -na 7 \
    -o xa1AHO.pdb \
    -P 4143 \
    -RP 3.0 \
    -RW 6.0 \
    -G 0.2 \
    -V 1 \
    ${XYZ} \
    ${ABC}
  if [ -e xa1AHO.pdb ] ; then
    echo "Part 3: (PROCEEDING) Acetate ions have been added; the latest"\
         "version of the"
    echo "        file is xa1AHO.pdb."
  else
    echo "ERROR >> Acetate ions should have been added, but the result"\
         "xa1AHO.pdb was not"
    echo "ERROR >> found."
    exit
  fi
fi

# This is the second AddToBox command, for adding ammonium.
if [ ! -e xm1AHO.pdb ] ; then
  echo "PART 3: (PROCEEDING) AddToBox output follows."
  ${AMBERHOME}/bin/AddToBox \
    -c xa1AHO.pdb \
    -a Ammonium.pdb \
    -na 3 \
    -o xm1AHO.pdb \
    -P 4192 \
    -RP 3.0 \
    -RW 6.0 \
    -G 0.2 \
    -V 1 \
    ${XYZ} \
    ${ABC}
  if [ -e xm1AHO.pdb ] ; then
    echo "Part 3: (PROCEEDING) Ammonium ions have been added; the latest "\
         "version of the"
    echo "        file is xm1AHO.pdb."
  else
    echo "ERROR >> Ammonium ions should have been added, but the result"\
         "xm1AHO.pdb was not"
    echo "ERROR >> found."
    exit
  fi
fi

echo "PART 3: (PROCEEDING) Finally, we need to add water to fill in the voids"\
     "between"
echo "        proteins and ions in the unit cell.  This will also be done with"
echo "        the AddToBox utility, but because the waters are intended to"\
     "fill in"
echo "        ANY space remaining in the unit cell they will be allowed to"\
     "come"
echo "        somewhat closer to other solvent particles already in the system."
echo "        Also, because there are already some waters in the system, the "\
     "total"
echo "        number added by AddToBox will be reduced accordingly.  In a real "
echo "        experiment, the exact number of waters needed would not be "\
     "known, and an"
echo "        iterative procedure of solvating the system and equilibrating it"\
     "in the"
echo "        constant pressure ensemble is recommended."

# This is the third and final AddToBox command, for adding water.
if [ ! -e solv1AHO.pdb ] ; then
  echo "PART 3: (PROCEEDING) AddToBox output follows."
  ${AMBERHOME}/bin/AddToBox \
    -c xa1AHO.pdb \
    -a spce.pdb \
    -na 179 \
    -o solv1AHO.pdb \
    -P 4207 \
    -RP 3.0 \
    -RW 3.0 \
    -G 0.2 \
    -V 1 \
    ${XYZ} \
    ${ABC}
  if [ -e solv1AHO.pdb ] ; then
    echo "Part 3: (COMPLETE) Water has been added; the latest version of the"\
         "file is"
    echo "        solv1AHO.pdb.  You probably noticed that AddToBox did not"
    echo "        \"achieve\" its goal of adding the specified number of"
         "waters over"
    echo "        several attempts.  This is normal operation.  The program "\
         "is designed,"
    echo "        and in this tutorial instructed, to recursively call itself"\
         "with"
    echo "        progressively lower clipping radii on solvent:solvent"\
         "placement."
    echo ""
  else
    echo "ERROR >> Water should have been added, but the result"\
         "solv1AHO.pdb was not"
    echo "ERROR >> found."
    exit
  fi
fi

# PART 4
echo "PART 4: A PDB file of the complete system, including crystallization "\
     "conditions,"
echo "        counterions, and solvent, has been created (solv1AHO.pdb).  The"\
     "next"
echo "        step is to create AMBER starting coordinate and topology files. "\
     "This"
echo "        will be done in the usual way, with the TLEAP program.  Be sure"\
     "to check"
echo "        the TLEAP input file (solvate.tleap), and note the use of the "
echo "        setBox, rather than solvateBox, command.  The initial coordinates"
echo "        produced by TLEAP will have the wrong box dimensions, but this"\
     "will be"
echo "        fixed immediately."

# Create the TLEAP input file.  Note the specification of the 16 disulfide bonds
# (four bonds per monomer), and the use of the setBox command, rather than the
# solvateBox command.  We use the setBox command because the system has already
# been solvated by AddToBox.
cat > solvate.tleap << EOF
source leaprc.ff99SB_spce
loadAmberPrep Acetate.prp
loadAmberPrep Ammonium.prp
ammparms = loadAmberParams Ammonium.frcmod
x = loadPdb "solv1AHO.pdb"
bond x.11.SG x.62.SG
bond x.15.SG x.35.SG
bond x.21.SG x.45.SG
bond x.25.SG x.47.SG
bond x.204.SG x.255.SG
bond x.208.SG x.228.SG
bond x.214.SG x.238.SG
bond x.218.SG x.240.SG
bond x.397.SG x.448.SG
bond x.401.SG x.421.SG
bond x.407.SG x.431.SG
bond x.411.SG x.433.SG
bond x.590.SG x.641.SG
bond x.594.SG x.614.SG
bond x.600.SG x.624.SG
bond x.604.SG x.626.SG
setBox x vdw 1.0
saveAmberParm x solv1AHO.top solv1AHO.crd
quit
EOF

# Run the TLEAP program to create the starting coordinates and topology
if [ ! -e ${TLEAP} ] ; then
  echo "ERROR >> The tleap program was not found."
  exit
fi
if [ ! -x ${TLEAP} ] ; then
  echo "ERROR >> The tleap program cannot be executed."
  exit
fi
${TLEAP} -f solvate.tleap > solvate.out
if [ -e solv1AHO.crd ] && [ -e solv1AHO.top ] ; then
  echo "PART 4: (PROCESSING) A topology (solv1AHO.top) and initial coordinates"
  echo "        (solv1AHO.crd) file have been prepared.  The final step is to"\
       "edit"
  echo "        the last line of the initial coordinates file to reflect the"\
       "unit cell"
  echo "        dimensions specified in the original PDB file, 1AHO.pdb."
else
  echo "ERROR >> Failure in TLEAP execution."
  exit
fi

# Run ChBox to edit the last line of the initial coordinates file so that it
# matches the true unit cell dimensions specified in 1AHO.pdb.
mv solv1AHO.crd tmp
if [ ! -e ${AMBERHOME}/bin/ChBox ] ; then
  echo "ERROR >> The ChBox program was not found."
fi
if [ ! -x ${AMBERHOME}/bin/ChBox ] ; then
  echo "ERROR >> The ChBox program cannot be executed."
  exit
fi
${AMBERHOME}/bin/ChBox -c tmp -o solv1AHO.crd ${XYZ} ${ABC}
if [ -e solv1AHO.crd ] ; then
  rm tmp
  echo "Part 4: (COMPLETE) At this point, solv1AHO.crd contains the initial"
  echo "        coordinates, and coorect initial unit cell dimensions, to"\
       "begin a"
  echo "        conventional molecular dynamics simulation of the 1AHO crystal"
  echo "        structure.  This tutorial is now complete."
  echo ""
else
  echo "ERROR >> Failure in ChBox execution."
fi
